/*  Board.cpp created Nov 5, 2010
 *
 *  Copyright (C) 2010 Alexey Abel
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 */

#include "Board.h"
#include <QGraphicsScene>
#include <QGraphicsView>
#include <QGraphicsPixmapItem> //TODO: only for debug?
#include <QGraphicsItem> //TODO: only for debug?
#include "Enemy.h"
#include "Time.h" //TODO: move to a more global scale
#include <QTimer> //TODO: only for debug
#include <QGraphicsGridLayout> //TODO: only for debug
#include <QGraphicsRectItem> //TODO: only for debug
#include <QGraphicsSceneMouseEvent> //TODO: only for debug
#include <QGraphicsItem> //TODO: only for debug
#include <QGraphicsItemGroup> //TODO: only for debug
#include "MapEditor.h"
#include <QLinkedList> //TODO: remove this if std containers used in fringe search later
#include <QHash> //TODO: remove this if std containers used in fringe search later
#include <typeinfo> //TODO: only debug


//TODO: make Board itself a type of QGraphicsScene, so there is not QWidget parent.
Board::Board(QWidget* parent) :
	QWidget(parent), columnCount_(td::columnCount()), rowCount_(td::rowCount()), goal_(CellCoordinate(0,0))
{
	editorActive_ = false;
	mapEditor_ = NULLP;

	// Create 2 dimensional list
	for (uint column=0; column<columnCount_; ++column)
	{
		cells_.append(QList <Cell> ());
		for (uint row=0; row<rowCount_; ++row)
		{
			cells_[column].append(Cell(column, row));
		}
	}

	setPalette(QPalette(QColor(152, 255, 17)));
	setAutoFillBackground(true);
	// Vertical size should be <(800-~60) because 800 is the resolution of some small nettops.
	setFixedSize(td::boardWidth(),td::boardHeight());


	scene_ = new QGraphicsScene(this);
	scene_->setItemIndexMethod(QGraphicsScene::NoIndex); //TODO: tune
	// Set scene dimension to board dimensions
	scene_->setSceneRect(QRectF(0,0,td::boardWidth(),td::boardHeight()));

//	QGraphicsRectItem* item = new QGraphicsRectItem(0,0,10,10);
//	qDebug() << item->boundingRect().width();
//	qDebug() << item->boundingRect().height();
//	qDebug() << item->boundingRect().topLeft().x();
//	qDebug() << item->boundingRect().bottomRight().x();
//	qDebug() << item->boundingRect().topLeft().y();
//	qDebug() << item->boundingRect().bottomRight().y();
//
//	qreal x1;
//	qreal y1;
//	qreal x2;
//	qreal y2;
//	qreal* px1 = &x1;
//	qreal* px2 = &x2;
//	qreal* py1 = &y1;
//	qreal* py2 = &y2;
//	item->boundingRect().getRect(px1,px2,py1,py2);
//	qDebug() << "coords:"<< *px1 << *px2 << *py1 << *py2;


	Time* time = new Time(); //TODO: move this up in the hirarchy
	connect(time, SIGNAL(step()), scene_, SLOT(advance()));
	time->setSpeed(1);

	//boardScene->removeItem(mapEditor);
	//delete mapEditor;



//	QGraphicsPixmapItem* item3 = boardScene->addPixmap(QPixmap("trollface.png"));
//	item3->setPos(100,100);

	QGraphicsView* boardView = new QGraphicsView (scene_, this);
	boardView->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
	boardView->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
	boardView->setOptimizationFlags(QGraphicsView::DontClipPainter
	                        | QGraphicsView::DontSavePainterState
	                        | QGraphicsView::DontAdjustForAntialiasing);
	//boardView->setRenderHint(QPainter::Antialiasing); //TODO: tune
    //boardView->setBackgroundBrush(QPixmap(":/images/cheese.jpg"));
    boardView->setCacheMode(QGraphicsView::CacheBackground); //TODO: tune
    boardView->setViewportUpdateMode(QGraphicsView::BoundingRectViewportUpdate); //TODO: tune
    boardView->setWindowTitle(QT_TRANSLATE_NOOP(QGraphicsView, "tower bla"));
    boardView->setContentsMargins(0,0,0,0);
    boardView->resize(QSize(td::boardWidth(),td::boardHeight()));
    boardView->setAutoFillBackground(true);
    boardView->setFrameStyle(0);
//    if (boardView->contentsMargins().isNull())
//    	qDebug() << "contentMargings von view ist null---";
//    qDebug() << "board width:"<< this->width();
//    qDebug() << "board height:"<< this->height();
//    qDebug() << "scene width:"<< boardScene->width();
//    qDebug() << "scene height:"<< boardScene->height();
//    qDebug() << "view width:"<< boardView->width();
//    qDebug() << "view height:"<< boardView->height();

    //TODO: assert that the following variables are set when game begins
    // - goal_
}

Board::~Board()
{
//	for (uint column=0; column<columnCount_; ++column) //only if QList is of type Cell*
//	{
//		for (uint row=0; row<rowCount_; ++row)
//		{
//			delete cells_[row][column];
//		}
//	}
}



Cell& Board::cell(uint column, uint row)
{
	//TODO: ASSERT legal index
	//TODO: this could be done with an iterator
	return cells_[column][row];
}



Cell& Board::cell(CellCoordinate cellcoordinate)
{
	//TODO: ASSERT legal index
	//TODO: this could be done with an iterator
	return cells_[cellcoordinate.column()][cellcoordinate.row()];
}


Cell const & Board::cell(CellCoordinate cellcoordinate) const
{
	//TODO: ASSERT legal index
	//TODO: this could be done with an iterator
	return cells_.at(cellcoordinate.column()).at(cellcoordinate.row());

}


uint Board::columnCount() const
{
	return columnCount_;
}



uint Board::rowCount() const
{
	return rowCount_;
}



CellCoordinate Board::start() const
{
	return start_;
}



CellCoordinate Board::goal() const
{
	return goal_;
}



void Board::setStart(CellCoordinate start)
{
	start_ = start;
}




void Board::setGoal(CellCoordinate goal)
{
	goal_ = goal;
}



void Board::addEnemy (td::EnemyType enemyType)
{
	Enemy* newEnemy = Enemy::create(*this, enemyType);
	scene_->addItem(newEnemy);
	//Enemy* newEnemy = dynamic_cast<Enemy*>(sceneElement2);
	newEnemy->setPos(15,15); //TODO: what happens if no pos set? 0,0?
	//newEnemy->calculatePath();
}


uint Board::manhattanLengthToGoal (CellCoordinate const & from) const
{
	return abs(from.column() - goal_.column())  +  abs(from.row() - goal_.row());
}



QLinkedList <Cell*> Board::walkableNeighborsOf(CellCoordinate center) //OPTIMIZATION?: precalculate walkable neighbors and save a list for each cell, update it when walkable is toggled anywhere
{
	QLinkedList <Cell *> walkableNeighbors;

	// order of neighbor check
	// 8 1 2
	// 7 X 3
	// 6 5 4

	// * O *
	// * X *
	// * * *

	if (center.row() > 0) //TODO: actually I have private access to cell_ because I am inside Board, why use cell()?
	{
		Cell* neighbor = &(cell(CellCoordinate(center.column(), center.row() - 1)));
		if (neighbor->isWalkable())
		{
			walkableNeighbors.push_back(neighbor);
		}
	}
	// * * O
	// * X *
	// * * *
	if (center.column() < (columnCount_ - 1) && center.row() > 0)
	{
		Cell* neighbor = &cell(CellCoordinate(center.column() + 1, center.row() - 1));
		if (neighbor->isWalkable())
		{
			walkableNeighbors.push_back(neighbor);
		}
	}
	// * * *
	// * X O
	// * * *
	if (center.column() < (columnCount_ - 1))
	{
		Cell* neighbor = &cell(CellCoordinate(center.column() + 1, center.row()));
		if (neighbor->isWalkable())
		{
			walkableNeighbors.push_back(neighbor);
		}
	}
	// * * *
	// * X *
	// * * O
	if (center.column() < (columnCount_ - 1)  && center.row() < (rowCount_ - 1))
	{
		Cell* neighbor = &cell(CellCoordinate(center.column() + 1, center.row() + 1));
		if (neighbor->isWalkable())
		{
			walkableNeighbors.push_back(neighbor);
		}
	}
	// * * *
	// * X *
	// * O *
	if (center.row() < (rowCount_ - 1))
	{
		Cell* neighbor = &cell(CellCoordinate(center.column(), center.row() + 1));
		if (neighbor->isWalkable())
		{
			walkableNeighbors.push_back(neighbor);
		}
	}
	// * * *
	// * X *
	// O * *
	if (center.column() > 0  && center.row() < (rowCount_ - 1))
	{
		Cell* neighbor = &cell(CellCoordinate(center.column() - 1, center.row() + 1));
		if (neighbor->isWalkable())
		{
			walkableNeighbors.push_back(neighbor);
		}
	}
	// * * *
	// O X *
	// * * *
	if (center.column() > 0)
	{
		Cell* neighbor = &cell(CellCoordinate(center.column() -1 , center.row()));
		if (neighbor->isWalkable())
		{
			walkableNeighbors.push_back(neighbor);
		}
	}
	// O * *
	// * X *
	// * * *
	if (center.column() > 0  && center.row() > 0)
	{
		Cell* neighbor = &cell(CellCoordinate(center.column() - 1, center.row() - 1));
		if (neighbor->isWalkable())
		{
			walkableNeighbors.push_back(neighbor);
		}
	}

	//returning list by value should be fast enough because of implicit sharing feature of Qt
	return walkableNeighbors;
}



QLinkedList<CellCoordinate> Board::pathToGoal(CellCoordinate start)
{
	return FringeSearchPath(start, goal_);
}



void Board::toggleEditor()
{
	if (editorActive_)
	{
		scene_->removeItem(mapEditor_);
		delete mapEditor_;
		mapEditor_ = NULLP;
		editorActive_ = false;
	}
	else
	{
		mapEditor_ = new MapEditor(*this);
		scene_->addItem(mapEditor_);
		editorActive_ = true;
	}
}


//from: http://www.cs.ualberta.ca/~games/pathfind/publications/cig2005.pdf
QLinkedList<CellCoordinate> Board::FringeSearchPath(CellCoordinate start, CellCoordinate goal)
{
	qDebug() << "fringe: entering";
	Cell * startCell = &cell(start); //TODO: is this legal? will the pointer remain valid?
	//TODO: try out std containers for perfomance!
	//TODO: change variable names to be more descriptive
	QLinkedList<Cell*> fringe; // now/later list?
	fringe.push_front(startCell);
	//    g,    parent
	//QHash<uint, Cell* > cache; //already in cells_

	QList< QList<Cell> >::iterator column;
	QList<Cell>::iterator row;


	//C[n] ← null for n != start
	//Also initialized during Cell construction
	for (column = cells_.begin(); column != cells_.end(); ++column)
	{
		for (row = cells_[0].begin(); row != cells_[0].end(); ++row)
			{
				row->g_ = 0; //TODO: better names for row? "cell"? //TODO: here it should be integer'last instead of 0?
				row->parent_ = NULLP;
			}
	}

	// unnecessary, see above loop
	//	start->g_ = 0;
	//	start->parent_ = NULLP;
	int fLimit = manhattanLengthToGoal(start); //probably should use "startCell" here
	bool found = false;
	qDebug() << "fringe: right before while loop";
	while (found == false && fringe.isEmpty() == false) //will be called at least once because found is set to false
	{
		qDebug() << "fringe: inside while loop";
		int fMin = 2147483647; // should be unlimited. choose last integer later? also make this unsigned?
		QLinkedList<Cell*>::iterator fringeIterator;
		for (fringeIterator=fringe.begin(); fringeIterator != fringe.end(); ++fringeIterator)
		{
			qDebug() << "fringe: inside outer for loop";
			//qDebug() << (*fringeIterator)->column() ;
			int gNode = (*fringeIterator)->g_;
			Cell* parent = (*fringeIterator)->parent_;

			int fNode = gNode + manhattanLengthToGoal((*fringeIterator)->coordinate());

			if (fNode > fLimit)
			{
				fMin = std::min(fNode,fMin);
				continue; //change this
			}
			if ((*fringeIterator)->coordinate() == goal_)
			{
				found = true;
				break;
			}
			// from here fNode <= fLimit and current node n is not goal
			QLinkedList <Cell*> successors = walkableNeighborsOf((*fringeIterator)->coordinate());
			QLinkedList <Cell*>::iterator successorsIterator;
			successorsIterator = successors.end();
			while (successorsIterator != successors.begin())
			{
				qDebug() << "fringe: inside successor while loop";
				--successorsIterator;

				 int gSum = gNode + (*successorsIterator)->g_;
				if ((*successorsIterator)->parent_ != NULLP)
				{
					//gSum       ~ new g value for successor
					//gSuccessor ~ old g value for successor
					int gSuccessor = (*successorsIterator)->g_;
					if (gSum >= gSuccessor)
					{
						// do nothing because new path is worse than old
						continue;
					}
				}
				// from here new path from n to s is better than what has been found so far
				// remove s from F
				if (fringe.contains(*successorsIterator))
				{
					fringe.removeOne(*successorsIterator);
				}
				// add s to F after n
				fringe.insert(fringeIterator + 1, (*successorsIterator));
				(*successorsIterator)->g_      = gSum;
				(*successorsIterator)->parent_ = (*fringeIterator);
			}
			fringe.removeOne(*fringeIterator);

		}
		fLimit = fMin;
	}

	QLinkedList<CellCoordinate> calculatedPath;

	if (found)
	{
		qDebug() << "fringe: found! constructing and returning path";
		Cell* c = &cell(goal);
		while (c->coordinate() != start) //whole loop works only if there is a connection between start and goal
		{
			calculatedPath.append(c->coordinate());
			c = c->parent_;
		}
	}
	else
	{
		qDebug() << "fringe: not found! returning empty list";
	}
	return calculatedPath;
}
