/*  Board.h created Nov 5, 2010
 *
 *  Copyright (C) 2010 Alexey Abel
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 */

#ifndef BOARD_H_
#define BOARD_H_

#include <QWidget>
#include "Td.h"
#include "Cell.h"
#include <QList>
#include <QStack>
#include <cmath>

class Enemy; //for the enemies_ QList
class MapEditor; //Baord.cpp includes MapEditor.h

//TODO:THINKING: Cell was made constant
class Board : public QWidget
{
	Q_OBJECT


public slots:
	void toggleEditor();
	void addEnemy (td::EnemyType enemyType);

public:
	Board(QWidget* parent = NULLP);
	virtual ~Board();
	Cell& cell(uint column, uint row);
	Cell& cell(CellCoordinate cellcoordinate);
	Cell const & cell(CellCoordinate cellcoordinate) const; //TODO: is this used?

	uint columnCount() const;
	uint rowCount()    const;
	CellCoordinate goal() const;
	CellCoordinate start() const;
	void setStart(CellCoordinate startCell);
	void setGoal(CellCoordinate goalCell);
	QLinkedList<CellCoordinate> pathToGoal(CellCoordinate start);


private:
	uint const columnCount_;
	uint const rowCount_;
	QList< QList<Cell> > cells_;
	QList<Enemy*> enemies_;
	CellCoordinate start_;//TODO: is this ok speedwise (not using a pointer)? probably no fast access is needed here.
	CellCoordinate goal_; //TODO: is this ok speedwise (not using a pointer)? probably no fast access is needed here.

	bool editorActive_;
	QGraphicsScene* scene_;
	MapEditor* mapEditor_;

	uint manhattanLengthToGoal (CellCoordinate const & from) const;//TODO: make this function take start and goal?
	QLinkedList <Cell*> walkableNeighborsOf(CellCoordinate center);

	QLinkedList<CellCoordinate> FringeSearchPath(CellCoordinate start, CellCoordinate goal);//TODO: add weight parameters later
};


#endif /* BOARD_H_ */
