/*  Cell.cpp created Nov 22, 2010
 *
 *  Copyright (C) 2010 Alexey Abel
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 */

#include "Cell.h"



// Only declared for "QStack path_" of Enemy, because it needs a default constructor.
// This should be only used to create invalid cells.
CellCoordinate::CellCoordinate() : column_(0), row_(0), valid_(false)
{
	//TODO: remove output, because it is ok now. This is the way to produce invalid cells for invalid references.
	qDebug() << "Calling default constructor CellCoordinate()!"; //TODO: this should never happen. catch this with exception?
}



CellCoordinate::CellCoordinate(uint column, uint row) : column_(column), row_(row)
{
	if (column_ < td::columnCount() && row_ < td::rowCount())
		valid_ = true;
	else
		valid_ = false;
	// check for (x > 0) is unnecessary because type is unsigned int
}



// This is needed to directly take pos() from mouse events, which returns a QPointF
CellCoordinate::CellCoordinate(QPointF pointF)
{
	// Will not work for negative coordinates because then
	// floor(-x) != int(-x)
	column_ = int((pointF.x()) - int(pointF.x()) % 10) / 10;
	row_    = int((pointF.y()) - int(pointF.y()) % 10) / 10;

	if (column_ < td::columnCount() && row_ < td::rowCount())
		valid_ = true;
	else
		valid_ = false;
	// check for (x > 0) is unnecessary because type is unsigned int
}



uint CellCoordinate::column() const
{
	if (valid_)
	{
		return column_;
	}
	else
	{
		//TODO: throw exception here
		qDebug() << "use of illegal cellcordinate";
		exit(1);
	}
}



uint CellCoordinate::row() const
{
	if (valid_)
	{
		return row_;
	}
	else
	{
		//TODO: throw exception here
		qDebug() << "use of illegal cellcordinate";
		exit(1);
	}
}



bool CellCoordinate::isValid() const
{
	return valid_;
}



QPointF CellCoordinate::toPointF() const
{
	//return QPointF(qreal(column()) * 10 + 5.0, qreal(row()) * 10 + 5.0);
	return QPointF(qreal(column()) * 10 , qreal(row()) * 10);
}



bool CellCoordinate::operator==(CellCoordinate const& right)
		{
			// Two invalid cells are equal.
			if (this->valid_ == false && right.valid_ == false)
			{
				return true;
			}
			// If only one is invalid they are not equal.
			// This is a XOR because false/false was handled by above.
			else if (this->valid_ == false || right.valid_ == false)
			{
				return false;
			}
			else
			{
				// Here both cells are valid
				if (this->column_ == right.column_ && this->row_ == right.row_)
				{
					return true;
				}
				return false;
			}
		}

bool CellCoordinate::operator!=(CellCoordinate const& right)
		{
			if (*this == right) return false;
			return true;
		}




Cell::~Cell()
{
	//delete representation_;

}



Cell::Cell(uint column, uint row) : walkable_(true), buildable_(true), column_(column), row_(row), g_(0), h_(-1), parent_(NULLP)
{
	// TODO Auto-generated constructor stub

}

//TODO: add this later when row and colum in cell were made constant
//Cell& Cell::operator=(Cell const& other)
//{
//	if (this != &other) {
//		//TODO: THIS CODE WAS NOT CHECKED!!!
//		Cell* newCell = new Cell(other);
//		delete &other;
////		Cell* newCell = new Cell(other.column(), other.row());
////		newCell->walkable_ = other.walkable_;
////		newCell->buildable_ = other.buildable_;
////		newCell->g_ = other.g_;
////		newCell->h_ = other.h_;
////		newCell->parent_ = other.parent_;
//		return *newCell;
//	}
//	return *this;
//}

void Cell::toggleWalkable() {
	walkable_ = !walkable_;
}



void Cell::toggleBuildable() {
	buildable_ = !buildable_;
}



void Cell::setWalkable(bool walkable) {
	walkable_ = walkable;
}



void Cell::setBuildable(bool buildable) {
	buildable_ = buildable;
}

bool Cell::isWalkable() const
{
	return walkable_;
}
bool Cell::isBuildable() const
{
	return buildable_;
}


uint Cell::column() const
{
	return column_;
}
uint Cell::row() const
{
	return row_;
}

CellCoordinate Cell::coordinate() const // TODO: check if this will be used in the end and delete if necas
{
	return CellCoordinate(column(), row());
}
