/*  Cell.h created Nov 22, 2010
 *
 *  Copyright (C) 2010 Alexey Abel
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 */

#ifndef CELL_H_
#define CELL_H_

#include "Td.h"
#include <QGraphicsRectItem> //TODO: maybe something more abstract, higher in class hirarchy?
#include <QPointF>


class CellCoordinate
{
public:
	CellCoordinate(); //only declared for "QStack path_"-member in Enemy class, because it needs a default constructor
	CellCoordinate(uint column, uint row);
	CellCoordinate(QPointF pointF);
	bool operator==(CellCoordinate const& right);
	bool operator!=(CellCoordinate const& right);
	uint column() const;
	uint row() const;
	bool isValid() const;
	QPointF toPointF() const;

private:
	uint column_;
	uint row_;
	bool valid_;
};


//	class InvalidCellCoordinateException: public std::exception //TODO: read proper manual and check this impemenation
//	{
//	  virtual const char* what() const throw()
//	  {
//	    return "Use of invalid CellCoordinates";
//	  }
//	};
/*
 *
 */
class Cell
{
public:
	virtual ~Cell();
	Cell(uint column, uint row);
	Cell& operator=(Cell const& other);

	uint column() const; //TODO: check if these are used
	uint row() const;
	CellCoordinate coordinate() const;

	void toggleWalkable();
	void toggleBuildable();
	void setWalkable(bool value);
	void setBuildable(bool value);
	bool isWalkable() const;
	bool isBuildable() const;

	//TODO: better solution than public members?
	// cost from the starting node to the current node
	int g_;
	// heuristic estimate of the distance from current node to goal
	int h_;
	// parent for pathfinding
	Cell* parent_;
	//for fringe TODO: optimize and give better names if possible
	bool visited_;
	bool inList_;
	bool validCache_;

private:
	bool walkable_;
	bool buildable_;
	uint column_; //should be constant but can't be because of QList
	uint row_; //should be constant but can't be because of QList
};

#endif /* CELL_H_ */
