/*  Enemy.cpp created Nov 5, 2010
 *
 *  Copyright (C) 2010 Alexey Abel
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 */


#include "Enemy.h"

bool Enemy::propertiesInitialized = false;//TODO: code here valid?
QHash<td::EnemyType, EnemyProperties> Enemy::properties;

Enemy* Enemy::create(Board& board, td::EnemyType type) { //TODO: second parameter as reference?
	if (!propertiesInitialized) {
		qDebug() << "initialisiere enemy properties";

		properties[td::square10px].name 		= QString("Square");
		properties[td::square10px].description 	= QString("I am a simple 10px square!");
		properties[td::square10px].skin 		= QPixmap("square10px.png"); //TODO: make path dynamic
		properties[td::square10px].health 		= 100.0;
		properties[td::square10px].speed  		= 2; //if speed is below 1, motion is not fluid
		properties[td::square10px].bounty 		= 33;

		properties[td::trollface].name 			= QString("Coolface");
		properties[td::trollface].description 	= QString("Problem officer? I doesn't afraid of anything!");
		properties[td::trollface].skin 			= QPixmap("coolface.png"); //TODO: make path dynamic
		properties[td::trollface].health 		= 100.0;
		properties[td::trollface].speed  		= 1; //if speed is below 1, motion is not fluid
		properties[td::trollface].bounty 		= 33;

		properties[td::rageface].name 			= QString("Rageface");
		properties[td::rageface].description 	= QString("THIS IS SPARTA!");
		properties[td::rageface].skin 			= QPixmap("rageface.png"); //TODO: make path dynamic
		properties[td::rageface].health 		= 100.0;
		properties[td::rageface].speed  		= 4; //if speed is below 1, motion is not fluid
		properties[td::rageface].bounty 		= 70;

		propertiesInitialized = true;
	}
	return new Enemy(board, type);
}

Enemy::Enemy(Board & board, td::EnemyType type) : QGraphicsPixmapItem(properties[type].skin) ,
									name_			(properties[type].name),
									description_	(properties[type].description),
									initialHealth_	(properties[type].health),
									initialSpeed_	(properties[type].speed),
									bounty_			(properties[type].bounty),
									skin_			(properties[type].skin),
									board_			(board),
									alive_			(true),
									health_			(initialHealth_),
									speed_			(initialSpeed_)

{
	//TODO: only for debugging, feed path from function later

	path_.push_back(CellCoordinate( 1 , 1 ));
	path_.push_back(CellCoordinate( 12 , 1 ));
	path_.push_back(CellCoordinate( 24 , 2 ));
	path_.push_back(CellCoordinate( 24 , 2 ));
	path_.push_back(CellCoordinate( 36 , 6 ));
	path_.push_back(CellCoordinate( 36 , 6 ));
	path_.push_back(CellCoordinate( 35 , 15 ));
	path_.push_back(CellCoordinate( 35 , 15 ));
	path_.push_back(CellCoordinate( 25 , 20 ));
	path_.push_back(CellCoordinate( 25 , 20 ));
	path_.push_back(CellCoordinate( 15 , 24 ));
	path_.push_back(CellCoordinate( 15 , 24 ));
	path_.push_back(CellCoordinate( 14 , 24 ));
	path_.push_back(CellCoordinate( 8 , 32 ));
	path_.push_back(CellCoordinate( 8 , 32 ));
	path_.push_back(CellCoordinate( 16 , 38 ));
	path_.push_back(CellCoordinate( 16 , 38 ));
	path_.push_back(CellCoordinate( 17 , 38 ));
	path_.push_back(CellCoordinate( 28 , 34 ));
	path_.push_back(CellCoordinate( 28 , 34 ));
	path_.push_back(CellCoordinate( 29 , 34 ));
	path_.push_back(CellCoordinate( 38 , 28 ));
	path_.push_back(CellCoordinate( 38 , 28 ));
	path_.push_back(CellCoordinate( 45 , 20 ));
	path_.push_back(CellCoordinate( 51 , 11 ));
	path_.push_back(CellCoordinate( 51 , 11 ));
	path_.push_back(CellCoordinate( 69 , 8 ));
	path_.push_back(CellCoordinate( 72 , 18 ));
	path_.push_back(CellCoordinate( 71 , 31 ));
	path_.push_back(CellCoordinate( 62 , 38 ));
	path_.push_back(CellCoordinate( 50 , 44 ));
	path_.push_back(CellCoordinate( 38 , 50 ));
	path_.push_back(CellCoordinate( 25 , 52 ));
	path_.push_back(CellCoordinate( 19 , 52 ));
	path_.push_back(CellCoordinate( 62 , 5 ));

	setPos(path_.takeFirst().toPointF());
}



Enemy::~Enemy()
{
	// TODO Auto-generated destructor stub
}



// game specific methods
// const getters
bool Enemy::isAlive() const
{
	return alive_;
}



qreal Enemy::health() const
{
	return health_;
}



uint Enemy::bounty() const
{
	return bounty_; //TODO:make a reference?
}



qreal Enemy::speed() const
{
	return speed_;
}



// setters
void Enemy::adjustHealth(qreal change)
{
	health_ += change;
	if (health_ <= 0)
	{
		//TODO: add signal emit
		alive_ = false;
	}
}


void Enemy::calculatePath() // TODO: add parameters for weighting here
{
	path_ = board_.pathToGoal(CellCoordinate(pos()));
}


void Enemy::advance(int step)
{
	if (step == 0) {
		if (!path_.isEmpty()) {
			// Because move needs a parameter (distance) it needs to be a separate function
			// Call with speed_. speed_ is distance/step.
			move(speed_);
		}
	}
}

void Enemy::move(qreal maxDistance)
{
	// Construct a line that goes from current position to next waypoint
	QLineF line = QLineF(pos(), path_.front().toPointF());

	//TODO: Delete this if Qt is fast enough, test with many units
//	QPointF nextWaypoint = line_.front().toPointF();
//	qreal dx = nextWaypoint.x() - pos().x();
//	qreal dy = nextWaypoint.y() - pos().y();
//	qreal hypotenuse = sqrt(dx*dx+dy*dy);

	// Calculate the part of this line that can be "walked" during this step.
	qreal part = maxDistance / line.length();
	//qDebug() << "part:" << part;


	if (part >= 1) {
		// This step's distance is bigger than (or the same as) the distance to the next waypoint.
		moveBy(line.dx() , line.dy());
		path_.removeFirst();
		//qDebug() << "part >=";
		// So we can (likely) continue from next waypoint in this step.
		if (part > 1)
		{
			if (!path_.isEmpty())
			{
				move(maxDistance - line.length());
			}
		}
	}
	// This step's distance is not enough to reach next waypoint.
	// Walk the appropriate part of the length.
	else // part < 1
	{
		moveBy(line.dx() * part, line.dy() * part);
		//qDebug() << "normal";
	}
}



EnemyProperties::EnemyProperties()
{
	// TODO: Auto-generated constructor stub

}



EnemyProperties::~EnemyProperties()
{
	// TODO: Auto-generated destructor stub
}
