/*  Enemy.h created Nov 5, 2010
 *
 *  Copyright (C) 2010 Alexey Abel
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 */

#ifndef ENEMY_H_
#define ENEMY_H_

#include <QGraphicsPixmapItem>
#include <QHash>
#include <QString>
#include <Td.h>
#include <QPixmap> //only for debug TODO
#include <QLinkedList>
#include "Cell.h"
#include "Board.h"
#include <cmath>


class EnemyProperties;
class Enemy: public QGraphicsPixmapItem //TODO: style: could be more generic (no Pixmap) //TODO: add start coordinates as parameters
{
public:
	// Separate create() is needed because enemy properties needs to be
	// initialized for base class constructor call.
	static Enemy* create(Board& board, td::EnemyType type);
	virtual ~Enemy();

	// game specific methods
	// const getters
	bool isAlive() const;
	qreal health() const;
	uint bounty() const;
	qreal speed() const;
	// modifiers
	void adjustHealth(qreal); //TODO: chose a name that

	// internal game mechanics
	void calculatePath();
	void advance(int step); //TODO: public?

private:
	// CONSTANT PROPERTIES
	//TODO: make all constant
	QString const & name_;
	QString const & description_;
	qreal const & initialHealth_;
	uint const &  initialSpeed_;
	uint const &  bounty_;

	QPixmap const & skin_; //TODO: move to properties array
	Board & board_; // can not be const because pathfinding changes cells_ of Board

	// VARIABLE PROPERTIES - must be placed after constant properties
	// because of object initialization in the constructor.
	bool alive_;
	qreal health_;
	qreal speed_;

	QLinkedList <CellCoordinate> path_; // probably this needs CellCoordinate() (default constructor)


	void move(qreal distance);
	Enemy(Board& board, td::EnemyType type);
	static bool propertiesInitialized;
	static QHash<td::EnemyType, EnemyProperties> properties;
	static bool initProperties (); //TODO: create definition
};



class EnemyProperties //TODO: change to struct? //TODO: aggregate this to Enemy
{
public:
	EnemyProperties();
	virtual ~EnemyProperties();

	QPixmap skin; //TODO: create new container for animation
	qreal health;
	qreal speed;
	uint bounty;
	QString name;
	QString description;
};


#endif /* ENEMY_H_ */
