/*  MapEditor.h created Nov 23, 2010
 *
 *  Copyright (C) 2010 Alexey Abel
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *
 */

#ifndef MAPEDITOR_H_
#define MAPEDITOR_H_

#include <QGraphicsItemGroup>
#include "Td.h"
#include "Cell.h"
#include <QGraphicsRectItem> //TODO: redundant, already in .h
#include "Board.h"

class MapEditorCell;



class MapEditor : public QGraphicsItemGroup
{
public:
	MapEditor(Board& board);
	virtual ~MapEditor();
	MapEditorCell& cell(uint column, uint row); // duplicate in Board.h. merge?
	MapEditorCell& cell(CellCoordinate cellcoordinate); // duplicate in Board.h. merge?
protected:
	void mousePressEvent   ( QGraphicsSceneMouseEvent * event );
	void mouseReleaseEvent ( QGraphicsSceneMouseEvent * event );
	void mouseMoveEvent    ( QGraphicsSceneMouseEvent * event );
	void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * event );
private:
//	std::vector< std::vector<Cell> >& cells_;
//	std::vector< std::vector<MapEditorCell> > editorMatrix_;
	// must be pointer to MapEditorCell because the container would need assignment parameter otherwise?
	QList< QList<MapEditorCell*> > editorMatrix_;
	Board& parentBoard_;
	enum MouseAction {walkableOn, walkableOff, buildableOn, buildableOff};
	MouseAction currentMouseAction_;
	CellCoordinate lastChangedCell_;

	void modifyCell (CellCoordinate  cell);

};



class MapEditorCell : public QGraphicsRectItem
{
public:
	virtual ~MapEditorCell();
	MapEditorCell(bool walkable = true, bool buildable = true);
	void toggleWalkable();
	void toggleBuildable();
	void setWalkable(bool walkable);
	void setBuildable(bool buildable);
	void setGoal(bool isGoal);
private:
	void updateColor();
	bool walkable_;
	bool buildable_;
	bool isGoal_;
};

#endif /* MAPEDITOR_H_ */
